@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
 from .const import MAX_REPRSTR
-from .util import limstr, kind, relative_url
-from .exceptions import NoMoreError, UnsupportedError
+from .util import limstr, kind, reddit_url
+from .exceptions import NoMoreError, UnexpectedResponse
 
 
 def identify_thing(dict_):
@@ -173,12 +173,12 @@ class Commentable(Thing):
         """
         return self._reddit._limit_get(self.permalink, limit=limit)[1]
     
-    def distinguish(self, how):
+    def distinguish(self, how=True):
         """Distinguishes this thing (POST).  Calls :meth:`narwal.Reddit.distinguish`.
         
         :param how: either True, False, or 'admin'
         """
-        return self._reddit.distinguish(self.name, how)
+        return self._reddit.distinguish(self.name, how=how)
     
     def delete(self):
         """Deletes this thing (POST).  Calls :meth:`narwal.Reddit.delete`.
@@ -213,6 +213,7 @@ class Reportable(Thing):
         """
         return self._reddit.report(self.name)
 
+
 class Listing(ListBlob):
     """A reddit :class:`Listing`.  See https://github.com/reddit/reddit/wiki/thing for more details.
     
@@ -261,16 +262,20 @@ class Listing(ListBlob):
         if self.after:
             return self._reddit._limit_get(self._path, params={'after': self.after}, limit=limit or self._limit)
         elif self._has_literally_more:
-            # this doesn't work.
-            #
-            # the only way to "load more comments" is to make a post to
-            # /api/morechildren; however, reddit returns a clusterfuck of a 
-            # response that contains the comments already formatted to be
-            # directly inserted into the HTML page.
-            #
-            # extracting the comment data requires more work than it's worth,
-            # to be honest.
-            raise UnsupportedError()
+            more = self[-1]
+            data = dict(
+                link_id=self[0].parent_id,
+                id=more.name,
+                children=','.join(more.children)
+            )
+            j = self._reddit.post('api', 'morechildren', data=data)
+            # since reddit is inconsistent here, we're hacking it to be
+            # consistent so it'll work with _thingify
+            d = j['json']
+            d['kind'] = 'Listing'
+            d['data']['children'] = d['data']['things']
+            del d['data']['things']
+            return self._reddit._thingify(d, path=self._path) 
         else:
             raise NoMoreError('no more items')
     
@@ -320,7 +325,7 @@ class Comment(Votable, Created, Commentable, Reportable):
         if relative:
             return u'/{0}'.format(r) 
         else:
-            return relative_url(r)
+            return reddit_url(r)
     
     @property
     def permalink(self):
@@ -389,6 +394,11 @@ class Link(Votable, Created, Commentable, Hideable, Reportable):
         """Approves this link (POST).  Calls :meth:`narwal.Reddit.approve`.
         """
         return self._reddit.approve(self.name)
+    
+    def refresh(self):
+        """Re-GETs this link (does not alter the object).  Returns :class:`Link` object.
+        """
+        return self._reddit.by_id(self.name)
 
 
 class Subreddit(Thing):
@@ -467,15 +477,44 @@ class Subreddit(Thing):
         """
         return self._reddit.submit_text(self.display_name, title, text)
     
-    def moderators(self):
+    def moderators(self, limit=None):
         """GETs moderators for this subreddit.  Calls :meth:`narwal.Reddit.moderators`.
+        
+        :param limit: max number of items to return
         """
-        return self._reddit.moderators(self.display_name)
+        return self._reddit.moderators(self.display_name, limit=limit)
     
-    def contributors(self):
-        """GETs contributors for this subreddit.  Calls :meth:`narwal.Reddit.contributors`.
+    def flair(self, name, text, css_class):
+        """Sets flair for `user` in this subreddit (POST).  Calls :meth:`narwal.Reddit.flairlist`.
+        
+        :param name: name of the user
+        :param text: flair text to assign
+        :param css_class: CSS class to assign to flair text
         """
-        return self._reddit.contributors(self.display_name)
+        return self._reddit.flair(self.display_name, name, text, css_class)
+    
+    def flairlist(self, limit=1000, after=None, before=None):
+        """GETs flairlist for this subreddit.  Calls :meth:`narwal.Reddit.flairlist`.
+        
+        :param limit: max number of items to return
+        :param after: full id of user to return entries after
+        :param before: full id of user to return entries *before*
+        """
+        return self._reddit.flairlist(self.display_name, limit=limit, after=after, before=before)
+    
+    def flaircsv(self, flair_csv):
+        """Bulk sets flair for users in this subreddit (POST).  Calls :meth:`narwal.Reddit.flaircsv`.
+        
+        :param flair_csv: CSV string
+        """
+        return self._reddit.flaircsv(self.display_name, flair_csv)
+    
+    def contributors(self, limit=None):
+        """GETs contributors for this subreddit.  Calls :meth:`narwal.Reddit.contributors`.
+        
+        :param limit: max number of items to return
+        """
+        return self._reddit.contributors(self.display_name, limit=limit)
 
 
 class Message(Created, Hideable, Reportable):
@@ -512,7 +551,7 @@ class Message(Created, Hideable, Reportable):
         return self._reddit.unread_message(self.name)
     
     def reply(self, text):
-        """POSTs reply to message with own message.
+        """POSTs reply to message with own message.  Returns posted message.
         
         URL: ``http://www.reddit.com/api/comment/``
         
@@ -523,7 +562,16 @@ class Message(Created, Hideable, Reportable):
             'id': '#commentreply_{0}'.format(self.name),
             'text': text,
         }
-        return self._reddit.post('api', 'comment', data=data)
+        j = self._reddit.post('api', 'comment', data=data)
+        try:
+            return self._reddit._thingify(j['json']['data']['things'][0], path=self._path)
+        except Exception:
+            raise UnexpectedResponse(j)
+    
+    def refresh(self):
+        """Re-GETs this message (does not alter the object).  Returns :class:`Message` object.
+        """
+        return self._reddit.get('message', 'messages', self.id)[0]
 
 
 class Account(Thing):
@@ -547,9 +595,9 @@ class Account(Thing):
         return unicode(self.name)
     
     def overview(self, limit=None):
-        """GETs overview of user's activities.  Returns :class:`Listing`.
+        """GETs overview of user's activities.  Calls :meth:`narwal.Reddit.user_overview`.
         
-        URL: http://www.reddit.com/user/<self.name>/overview/
+        :param limit: max number of items to get
         """
         return self._reddit.user_overview(self.name, limit=limit)
     
@@ -561,11 +609,11 @@ class Account(Thing):
         return self._reddit.user_comments(self.name, limit=limit)
     
     def submitted(self, limit=None):
-        """GETs user's submissions.  Calls :meth:`narwal.Reddit.user_comments`.
+        """GETs user's submissions.  Calls :meth:`narwal.Reddit.user_submitted`.
         
         :param limit: max number of submissions to get
         """
-        return self._reddit._limit_get('user', self.name, 'submitted', limit=limit)
+        return self._reddit.user_submitted(self.name, limit=limit)
     
     def about(self):
         """GETs this user (again).  Calls :meth:`narwal.Reddit.user`.
